"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const bluebird_1 = require("bluebird");
const lodash_1 = require("lodash");
class VersioningPluginBuilder {
    constructor(supportedApplications, supportedVersions) {
        this.supportedVersions = supportedVersions;
        this.supportedApplications = supportedApplications;
        this.adapters = new Map();
    }
    addAdapter(version, adapter) {
        this.validateAdapterVersion(version);
        this.validateAdapter(adapter);
        const versions = this.convertVersionToSet(version);
        versions.forEach(vs => {
            this.validateVersion(vs);
            this.adapters.set(vs, adapter);
        });
        return this;
    }
    /**
     * Build a RequestHandler plugin that will
     * @returns {IRequestHandlerPlugin} A RequestHandlerPlugin
     */
    getPlugin() {
        return (requestHandler) => {
            this.setPreHook(requestHandler);
            this.setPostHook(requestHandler);
        };
    }
    convertVersionToSet(version) {
        const versions = 'string' === typeof version ? [version] : version;
        return new Set(versions);
    }
    /**
     * Build a post request handler hook
     * @param {RequestHandler} requestHandler
     */
    setPreHook(requestHandler) {
        requestHandler.pre((req, res, handler) => {
            return bluebird_1.Promise.try(() => {
                const adapter = this.getAdapter(req);
                if (null !== adapter && 'function' === typeof adapter.onPreHandler) {
                    return adapter.onPreHandler(req, res, handler);
                }
            });
        });
    }
    /**
     * Build a post request handler hook
     * @param {RequestHandler} requestHandler
     */
    setPostHook(requestHandler) {
        requestHandler.post((req, res, handler) => {
            return bluebird_1.Promise.try(() => {
                const adapter = this.getAdapter(req);
                if (null !== adapter && 'function' === typeof adapter.onPostHandler) {
                    return adapter.onPostHandler(req, res, handler);
                }
            });
        });
    }
    /**
     * Lookup for an adapter based on the version found in Accept header
     *
     * @param {Request} req
     * @returns {IVersionAdapter} Version adapter
     */
    getAdapter(req) {
        const version = this.parseVersion(req);
        if ('string' !== typeof version) {
            return null;
        }
        return this.adapters.get(version) || null;
    }
    /**
     * Parse version from request.
     * Look up version into the "Accept" header and try to match the version against
     * VersioningPluginBuilder.VERSION_REGEX
     *
     * @param {Request} req
     * @returns {string|null} Returns the parsed version or null if no version is found
     */
    parseVersion(req) {
        const acceptHeader = req.get('Accept');
        if ('string' !== typeof acceptHeader) {
            return null;
        }
        const result = VersioningPluginBuilder.VERSION_REGEX.exec(acceptHeader);
        if (!Array.isArray(result)) {
            return null;
        }
        if (!this.supportedApplications.has(String(result[1]))) {
            return null;
        }
        return String(result[2]);
    }
    /**
     * Validate an adapter
     * Expected adapter to be an object with versions
     *
     * @param {IVersionAdapter} adapter
     * @throws {TypeError}
     */
    validateAdapter(adapter) {
        if (!lodash_1.isObject(adapter)) {
            throw new TypeError('Expected adapter to be an object');
        }
    }
    /**
     * Check if the version is supported and if there are no adapters already set for this version
     *
     * @param {string} version
     * @throws {TypeError}
     */
    validateVersion(version) {
        if (!this.supportedVersions.has(version)) {
            throw new TypeError(`Version ${version} is not an acceptable version, expected one of [${Array.from(this.supportedVersions).join(', ')}]`);
        }
        if (this.adapters.has(version)) {
            throw new TypeError(`An adapter has aready been registered for version "${version}"`);
        }
    }
    /**
     * Validate a list of versions
     * Expect versions to be a non empty Set
     *
     * @param {Set<string>} versions
     * @throws {TypeError}
     */
    validateAdapterVersion(versions) {
        const hasValidType = (Array.isArray(versions) &&
            versions.every(version => 'string' === typeof version)) ||
            'string' === typeof versions;
        if (!hasValidType) {
            throw new TypeError('Expected version to be either a string or an array of strings');
        }
        if (versions.length === 0) {
            throw new TypeError('Expected version not to be empty');
        }
    }
}
VersioningPluginBuilder.VERSION_REGEX = /application\/vnd\.([^+]+)\+[^;]+;\s*version=([^\s]+)/;
exports.VersioningPluginBuilder = VersioningPluginBuilder;
//# sourceMappingURL=VersioningPluginBuilder.js.map